/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/

#include <firestorm.h>
#include <args.h>
#include <plugin.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>

/* GRE: Generic routing encapsulation */
void gre_decode(struct packet *);
struct proto gre_p=init_proto("gre", gre_decode, NULL);
struct proto_req gre_r[]={
	proto_request("ip", 47),
	null_request()
};

void gre_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen];

	if ( (p->layer[p->llen+1].h.raw=
		l->h.raw+sizeof(struct pkt_grehdr))
		> p->end ) goto finish;

	p->llen++;

	if ( p->llen >= PKT_LAYERS ) {
		dispatch(p);
		return;
	}

	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.gre->proto == pc->id ) {
			p->layer[p->llen].flags=0;
			p->layer[p->llen].session=NULL;
			p->layer[p->llen].proto=pc->proto;
			pc->proto->decode(p);
			return;
		}
	}

finish:
	if ( p->layer[p->llen].h.raw<p->end )
		p->layer[p->llen++].proto=NULL;
	dispatch(p);
}

struct proto_table firestorm_protocols[]={
	{.proto = &gre_p, .preq = gre_r},
	{.proto = NULL}
};

struct plugin_desc firestorm_plugin_desc = {
	.size		= sizeof(firestorm_plugin_desc),
	.name		= "decode.gre",
	.desc		= "Generic Routing Encapsulation",
	.author_name	= "Gianni Tedesco",
	.author_email	= "gianni@scaramanga.co.uk",
	.ver_major	= 2,
	.ver_minor	= 0,
};
