/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include "tcpip.h"

static struct {
	char *name;
	unsigned int val;
}ipopts[]={
	{"eol",		IPOPT_EOL},
	{"nop",		IPOPT_NOP},
	{"rr",		IPOPT_RR},
	{"rtralt",	IPOPT_RTRALT},
	{"ts",		IPOPT_TS},
	{"sec",		IPOPT_SECURITY},
	{"lsrr",	IPOPT_LSRR},
	{"lsrre",	IPOPT_LSRR_E},
	{"satid",	IPOPT_SATID},
	{"ssrr",	IPOPT_SSRR},
	{"any",		0xff},
	{NULL, 0}
};

static u_int8_t lookup[0x100]={
	[IPOPT_EOL] 1,
	[IPOPT_NOP] 2,
	[IPOPT_RR] 3,
	[IPOPT_RTRALT] 4,
	[IPOPT_TS] 5,
	[IPOPT_SECURITY] 6,
	[IPOPT_LSRR] 7,
	[IPOPT_LSRR_E] 8,
	[IPOPT_SATID] 9,
	[IPOPT_SSRR] 10,
	[0xff] 0,
};

#define NUM_VALID 11

struct ipopt_detect {
	struct detect d;
	struct detect *result[NUM_VALID];
	struct alert *alert[NUM_VALID];
	struct detect *any;
};

static u_int32_t ipopts_get(struct packet *p, int l)
{
	struct pkt_iphdr *iph=p->layer[l].h.ip;
	unsigned char *tmp, *end;
	u_int32_t ret = 0;

	if (iph->ihl == 5 )
		return ret;

	end=tmp=(unsigned char *)iph;
	tmp+=20;
	end+=iph->ihl<<2;

	if ( end < (unsigned char *)p->end )
		end = (unsigned char *)p->end;

	while ( tmp < end ) {
		u_int32_t this = lookup[*tmp];

		switch ( *tmp ) {
			case IPOPT_EOL:
				goto err;
			case IPOPT_NOP:
			case IPOPT_RTRALT:
				tmp++;
				break;
			case IPOPT_SECURITY:
				tmp+=11;
				break;
			case IPOPT_SATID:
				tmp+=4;
				break;
			default:
				if ( ++tmp < end ) {
					tmp+=*tmp;
					break;
				}else goto err;
		}

		ret |= (1<<this);
		tmp++;
	}

err:
	return ret;
}

static void ipopt_match(struct detect *d, struct packet *p, int l)
{
	struct ipopt_detect *x = (struct ipopt_detect *)d;
	u_int32_t opts;
	u_int32_t i;

	opts = ipopts_get(p, l);
	if ( opts == 0 )
		goto skip;

	if ( x->alert[0] )
		alert_tag(p, x->alert[0], l);
	if ( x->result[0] )
		x->result[0]->match(x->result[0], p, l);

	for(i=1; i < NUM_VALID; i++) {
		if ( opts & (1<<i) ) {
			if ( x->alert[i] )
				alert_tag(p, x->alert[i], l);
			if ( x->result[i] )
				x->result[i]->match(x->result[i], p, l);
		}
	}

skip:
	if ( x->any )
		x->any->match(x->any, p, l);
}

static struct detect *ipopt_new(struct matcher *m)
{
	struct ipopt_detect *x;

	x = calloc(1, sizeof(*x));
	if ( x == NULL )
		return NULL;

	x->d.match = ipopt_match;

	return (struct detect *)x;
}

static struct detect **ipopt_default(struct detect *d)
{
	struct ipopt_detect *x = (struct ipopt_detect *)d;
	return &x->any;
}

static struct detect **ipopt_insert(struct detect *d, struct sig_node *n)
{
	struct ipopt_detect *x = (struct ipopt_detect *)d;
	unsigned long val = (unsigned long)n->private;
	int idx = lookup[val];

	if ( n->negate ) {
		mesg(M_ERR, "ipopt negation not yet supported");
		return NULL;
	}

	x->alert[idx] = alert_best(x->alert[idx], n->alert);

	return &x->result[idx];
}

/* IP Options */
static int ipopt_validate(struct matcher *match, char *args,
			void **priv, struct modifier *m)
{
	int i;
	char *tmp;

	if ( !args )
		return 0;

	/* Strip trailing and leading whitespace */
	while ( *args==' ' || *args=='\t' )
		args++;

	for(tmp=args; *tmp; tmp++) {
		if ( *tmp==' ' || *tmp=='\t' ) {
			*tmp=0;
			break;
		}
	}

	for(i=0; ipopts[i].name; i++) {
		if ( !strcasecmp(args, ipopts[i].name) ) {
			*(unsigned int *)priv = ipopts[i].val;
			return 1;
		}
	}

	return 0;
}

struct matcher ip_opts_matcher = {
	.validate = ipopt_validate,
	.new_node = ipopt_new,
	.insert_node = ipopt_insert,
	.insert_default_node = ipopt_default,
	.free_priv = free,
};
