/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include "tcpip.h"

#define MODE_NORMAL	0
#define MODE_ALL	1
#define MODE_NOT	2
#define MODE_ANY	3

struct flags_node {
	u_int8_t flags;
	u_int8_t mode;
	int negate;
	struct alert *alert;
	struct detect *result;
	struct flags_node *next;
};

struct flags_detect {
	struct detect d;
	struct flags_node *nodes;
	struct detect *any;
};

static void flags_match(struct detect *d, struct packet *p, int l)
{
	struct flags_detect *x = (struct flags_detect *)d;
	struct flags_node *n;
	u_int8_t flags;

	flags = p->layer[l+1].h.tcp->flags.flags;

	for(n=x->nodes; n; n=n->next ) {
		int ret=0;

		switch ( n->mode ) {
		case MODE_NORMAL:
			ret = n->negate ^ (flags == n->flags);
			break;
		case MODE_ANY:
			ret = n->negate ^ ((flags & n->flags) != 0);
			break;
		case MODE_ALL:
			ret = n->negate ^ ((flags & n->flags) == n->flags);
			break;
		case MODE_NOT:
			ret = n->negate ^ ((flags & n->flags) == 0);
			break;
		}

		if ( ret ) {
			if ( n->alert )
				alert_tag(p, n->alert, l);
			if ( n->result )
				n->result->match(n->result, p, l);
		}
	}

	if ( x->any )
		x->any->match(x->any, p, l);
}

static struct detect *flags_new(struct matcher *m)
{
	struct flags_detect *x;

	x = calloc(1, sizeof(*x));
	if ( x == NULL )
		return NULL;

	x->d.match = flags_match;

	return (struct detect *)x;
}

static struct detect **flags_default(struct detect *d)
{
	struct flags_detect *x = (struct flags_detect *)d;
	return &x->any;
}

static struct detect **flags_insert(struct detect *d, struct sig_node *n)
{
	struct flags_detect *x = (struct flags_detect *)d;
	struct flags_node *p = (struct flags_node *)n->private;
	struct flags_node *tmp;

	p->alert = n->alert;
	p->negate = n->negate;

	for(tmp=x->nodes; tmp; tmp=tmp->next) {
		if ( p->flags != tmp->flags )
			continue;
		if ( p->mode != tmp->mode )
			continue;
		if ( p->negate != tmp->negate )
			continue;

		tmp->alert = alert_best(tmp->alert, p->alert);
		n->match->free_priv(n->private);
		return &tmp->result;
	}

	p->next = x->nodes;
	x->nodes = p;

	return &p->result;
}

static int flags_validate(struct matcher *match, char *args,
			void **priv, struct modifier *m)
{
	struct flags_node *p;
	char *tmp;

	if ( !args )
		return 0;
	
	if ( !(p=calloc(1, sizeof(*p))) )
		return 0;

	for(tmp=args; *tmp; tmp++) {
		switch(*tmp) {
		case 'f':
		case 'F':
			p->flags |=  TCP_FIN;
			break;
		case 's':
		case 'S':
			p->flags |= TCP_SYN;
			break;
		case 'r':
		case 'R':
			p->flags |= TCP_RST;
			break;
		case 'p':
		case 'P':
			p->flags |= TCP_PSH;
			break;
		case 'a':
		case 'A':
			p->flags |= TCP_ACK;
			break;
		case 'u':
		case 'U':
			p->flags |= TCP_URG;
			break;
		case 'E':
		case 'e':
		case '2':
			p->flags |= TCP_ECE;
			break;
		case 'C':
		case 'c':
		case '1':
			p->flags |= TCP_CWR;
			break;
		case '0':
			p->flags = 0;
			break;
		case '!':
			p->mode = MODE_NOT;
			break;
		case '*':
			p->mode = MODE_ANY;
			break;

		case '+':
			p->mode = MODE_ALL;
			break;
		case ' ':
		case '\t':
		case ',':
			break;
		default:
			free(p);
			return 0;
		}
	}

	*priv = p;

	return 1;
}

struct matcher tcp_flags_matcher = {
	.validate = flags_validate,
	.new_node = flags_new,
	.insert_node = flags_insert,
	.insert_default_node = flags_default,
	.free_priv = free,
};
