/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include <firestorm.h>
#include <args.h>
#include <plugin.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>
#include <netinet/in.h>

/* SAP: Novell's Service Advertising Protocol */
int sap_print(struct packet *p, int l, char *buf, size_t buflen);
void sap_decode(struct packet *p);
struct proto sap_p=init_proto("sap", sap_decode, sap_print);
struct proto_req sap_r[]={
        proto_request("ipx", __constant_htons(0x0452)),
	null_request()
};

int sap_print(struct packet *p, int l, char *buf, size_t buflen)
{
	struct pkt_sap *s = p->layer[l].h.sap;
	char *type="Unknown";
	char sname[49];

	switch (htons(s->op)) {
		case 1:
		case 2:
			type="General service";
			break;
		case 3:
		case 4:
			type="Nearest service";
			break;
	};
	if ((htons(s->op) % 2) == 0) {
		strncpy(sname, s->server_name, sizeof(sname));
		sname[sizeof(sname)-1]='\0';
		return snprintf(buf,buflen,"%s reponse server=%s",type,sname);
	} else {
		return snprintf(buf,buflen,"%s query",type);
	}
}

void sap_decode(struct packet *p)
{
	struct layer *l=&p->layer[p->llen];

	if ( l->h.raw+sizeof(struct pkt_sap) > p->end ) {
		dispatch(p);
		return;
	}

	p->llen++;
	dispatch(p);
}

struct proto_table firestorm_protocols[]={
	{.proto = &sap_p, .preq = sap_r},
	{.proto = NULL}
};

struct plugin_desc firestorm_plugin_desc = {
	.size		= sizeof(firestorm_plugin_desc),
	.name		= "decode.sap",
	.desc		= "Service Advertising Protocol",
	.author_name	= "John Leach",
	.author_email	= "john@johnleach.co.uk",
	.ver_major	= 0,
	.ver_minor	= 2,
};
