/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/

#include "tcpip.h"

static struct sub_table udp_subs[]={
	{"src", "ip.src"},
	{"dst", "ip.dst"},
	{"tos", "ip.tos"},
	{"id", "ip.id"},
	{"fragoffset", "ip.fragofs"},
	{"fragbits", "ip.fragbits"},
	{"ttl", "ip.ttl"},
	{"ip_proto", "ip.proto"},
	{"sameip", "ip.sameip"},
	{"ipopts", "ip.opts"},
	{"sport", "udp.sport"},
	{"dport", "udp.dport"},
	{NULL, NULL},
};

struct proto udp_p={
	.name		= "udp",
	.decode		= udp_decode,
	.print		= udp_dprint,
	.sig_add	= udp_commit,
	.sig_match	= udp_match,
	.sig_subs	= udp_subs,
	.fields		= udp_fields,
	.tag		= udp_tag,
};

struct proto_req udp_r[]={
	proto_request("ip", 17),
	null_request()
};

/* Generator */
struct generator udp_gen=init_generator("sig.udp", NULL);

void udp_tag(struct packet *p, int l, struct alert_tag *t)
{
	mesg(M_DEBUG, "UDP packet tagged");
}

int udp_dprint(struct packet *p, int l, char *buf, size_t buflen)
{
	struct layer *a = &p->layer[l];
	return snprintf(buf, buflen,
		"%u > %u len=%u csum=0x%x",
		htons(a->h.udp->sport),
		htons(a->h.udp->dport),
		htons(a->h.udp->len),
		htons(a->h.udp->csum));
}

void udp_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen];

	if ( (p->layer[p->llen+1].h.raw=
		l->h.raw+sizeof(struct pkt_udphdr))
		> p->end ) goto finish;

	p->llen++;

	if ( p->llen >= PKT_LAYERS ) {
		dispatch(p);
		return;
	}

	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.udp->dport == pc->id ||
			l->h.udp->sport == pc->id ) {
			p->layer[p->llen].flags=0;
			p->layer[p->llen].session=NULL;
			p->layer[p->llen].proto=pc->proto;
			pc->proto->decode(p);
			return;
		}
	}

finish:
	if ( p->layer[p->llen].h.raw<p->end )
		p->layer[p->llen++].proto=NULL;
	dispatch(p);
}


/* ===================================================
 * PACKET MATCHING STUFF BEYOND THIS POINT
 * ===================================================
 */

static struct detect_root udp_root;

void udp_match(struct packet *p, int l)
{
	detect(&udp_root, p, --l);
}

int udp_commit(struct rule *r)
{
	struct alert *a;
	struct sig_node *x;

	x = detect_new_sig(r, NULL);
	if ( x == NULL )
		return 0;

	a = calloc(1, sizeof(*a));
	if ( a == NULL ) {
		detect_free_sig(x, r->num_criteria);
		return 0;
	}

	memcpy(a, &r->alert, sizeof(*a));
	a->gen = &udp_gen;

	return detect_add_sig(&udp_root, x, r->num_criteria, a);
}
