/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/

#include <firestorm.h>

#include <sys/socket.h>
#include <netinet/in.h>

#include <args.h>
#include <plugin.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>

void vlan_decode(struct packet *);
int vlan_dprint(struct packet *, int, char *, size_t );

static struct proto vlan_p=init_proto("802.1q", vlan_decode, vlan_dprint);
static struct proto_req vlan_r[]={
	/* XXX: Do not add any other relations here!! */
	proto_request("ethernet", __constant_htons(0x8100)),
	null_request()
};


int vlan_dprint(struct packet *p, int l, char *buf, size_t buflen)
{
	u_int16_t vlan, prio, cfi;

	vlan = ntohs(p->layer[l].h.vlan->vlan);

	prio = vlan & 0xe000;
	cfi = vlan & 0x2000;
	vlan = vlan & 0x0fff;

	return snprintf(buf, buflen, "vlan=%u priority=%u cfi=%u proto=0x%x",
		vlan, prio, cfi,
		ntohs(p->layer[l].h.vlan->proto));
}

/* XXX: Cannot be at index 0 in the layer array! */
void vlan_decode(struct packet *p)
{
	struct proto_child *pc=p->layer[p->llen-1].proto->children;
	struct layer *l=&p->layer[p->llen];

	/* Fill in the next layers pointer */
	if ( (p->layer[p->llen+1].h.raw=
		l->h.raw+sizeof(struct pkt_vlanhdr))
		> p->end ) goto finish;

	p->llen++;

	if ( p->llen >= PKT_LAYERS ) {
		dispatch(p);
		return;
	}

	/* Find a relevent child */
	for(;pc; pc=pc->next)
	{
		if ( l->h.vlan->proto == pc->id ) {
			p->layer[p->llen].flags=0;
			p->layer[p->llen].session=NULL;
			p->layer[p->llen].proto=pc->proto;
			pc->proto->decode(p);
			return;
		}
	}

	/* Just data */
finish:
	if ( p->layer[p->llen].h.raw<p->end )
		p->layer[p->llen++].proto=NULL;
	dispatch(p);
}

struct proto_table firestorm_protocols[]={
	{.proto = &vlan_p, .preq = vlan_r},
	{.proto = NULL}
};

struct plugin_desc firestorm_plugin_desc = {
	.size		= sizeof(firestorm_plugin_desc),
	.name		= "decode.vlan",
	.desc		= "802.1q aka vlan",
	.author_name	= "Gianni Tedesco",
	.author_email	= "gianni@scaramanga.co.uk",
	.ver_major	= 1,
	.ver_minor	= 0,
};
