/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __ARGS_HEADER_INCLUDED__
#define __ARGS_HEADER_INCLUDED__

#define AKEY_SIZE 128
#define AVAL_SIZE 2048

#define ARGTYPE_NOP		0
#define ARGTYPE_UINT		1
#define ARGTYPE_PUINT		2
#define ARGTYPE_INT		3
#define ARGTYPE_PINT		4
#define ARGTYPE_BYTES		5
#define ARGTYPE_PBYTES		6
#define ARGTYPE_STRING		7
#define ARGTYPE_BOOL		8
#define ARGTYPE_PBOOL		9
#define ARGTYPE_DOUBLE		10
#define ARGTYPE_PDOUBLE		11
#define ARGTYPE_OFF		12
#define ARGTYPE_POFF		13
#define ARGTYPE_FIBUF		14
#define ARGTYPE_FOBUF		15
#define ARGTYPE_MAX		16

#define ARGTYPE_LITERAL		ARGTYPE_STRING

struct arg;

typedef int(*arg_cbfn_t)(struct arg *, void *);

struct arg {
	char *name;
	unsigned int type;

	arg_cbfn_t fn;

	union {
		char *v_str;
		char *v_literal;
		unsigned int v_bytes;
		unsigned int v_uint;
		unsigned int v_bool;
		double v_double;
		off_t v_off;
		int v_int;
		unsigned int *vp_bytes;
		unsigned int *vp_uint;
		unsigned int *vp_bool;
		double *vp_double;
		off_t *vp_off;
		int *vp_int;
		struct fibuf *fibuf;
		struct fobuf *fobuf;
	}val;

	const char *help;
};

int arg_dispatch(struct arg *a, const char *key, const char *val, void *priv)
	__nonull(1,2,3);
int args_parse(struct arg *a, const char *str, void *) __nonull(1,2);
const char *args_typename(unsigned int type);

#endif /* __ARGS_HEADER_INCLUDED__ */
