/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __CAPTURE_HEADER_INCLUDED__
#define __CAPTURE_HEADER_INCLUDED__

#include <signal.h> /* For sig_atomic_t */

#define CAP_STATE_NULL		0 /* Not started */
#define CAP_STATE_ENDED		0 /* Finished */
#define CAP_STATE_INIT		1 /* Initialised */
#define CAP_STATE_CAPTURE	2 /* mid-capture */
#define CAP_STATE_STOP		3 /* Stopping */
#define CAP_STATE_INTR		4 /* Interrupted */

/* CAP_STATE_INT interrupt types */
#define CAP_INTR_UNKNOWN	(1<<0)
#define CAP_INTR_USR1		(1<<1)
#define CAP_INTR_USR2		(1<<2)
#define CAP_INTR_HUP		(1<<3)
#define CAP_INTR_TIMER		(1<<4)
#define CAP_INTR_DEADLY		(1<<5)

struct capture {
	struct capdev		*capdev;
	volatile sig_atomic_t	state;
	const char		*args;
	void			*priv;
	volatile sig_atomic_t	intr; /* interrupt type */
};

typedef int (*proc_capdev_init)(const char *, struct capture *, void **)
	__nonull(2,3);
typedef void (*proc_capdev_end)(void *);
typedef void (*proc_capdev_go)(void *, struct capture *);

struct capdev {
	struct capdev		*next;
	const char		*name;
	const char		*desc;
	proc_capdev_init	init;
	proc_capdev_end		end;
	proc_capdev_go		go;
	int			live;
};

void capture_setup(const char *, const char *) __nonull(1);
void capture_go(void);
void capture_interrupt(int);
int capture_is_live(void);
void capdev_load(void);
struct __private capdev *capdev_find(const char *) __nonull(1);

void capdev_init_packet(struct packet *p, struct capture *cap)
			__nonull(1,2);

static inline u_int32_t capdev_endian_flag(int swapped)
{
#if HOST_ORDER == FP_BIG_ENDIAN
	if ( !swapped )
#else
	if ( swapped )
#endif
		return FP_BIG_ENDIAN;
	return 0;
}


void serial_number(struct packet *) __nonull(1);

#endif /* __CAPTURE_HEADER_INCLUDED__ */
