/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __DECODE_HEADER_INCLUDED__
#define __DECODE_HEADER_INCLUDED__

#define PROTO_LEN 15

typedef unsigned int proto_id_t;

struct proto_child {
	struct proto_child	*next;
	proto_id_t		id;
	struct proto		*proto;
};

#define proto_request(x, y) {x, y}
#define null_request() {NULL, 0}
struct proto_req {
	char			*name;
	proto_id_t		id;
};

/* temporary structure used internally */
struct preq_list {
	struct preq_list	*next;
	struct proto		*proto;
	struct proto_req	*arr;
};

/* decode a packet */
typedef void (*proc_decode)(struct packet *);
typedef size_t (*proc_sdecode)(struct packet *, char *, size_t);

/* print it in human-readable form */
typedef int  (*proc_dprint)(struct packet *, int, char *, size_t);

/* signature engine */
typedef int  (*proc_sig_add)(struct rule *);
typedef void (*proc_sig_match)(struct packet *, int);

/* serialize/deserialize session data for file storage */
typedef int  (*proc_serialize)(struct packet*, int, char *, size_t);
typedef void *(*proc_deserialize)(struct packet*, int, const char *, size_t);
typedef void (*proc_session_free)(void *);

/* Tagging support */
struct alert_tag;
typedef void(*proc_tag)(struct packet *p, int l, struct alert_tag *t);

/* struct proto, and some initializer macros */
#define init_proto(x,y,z)        { .name=x, .decode=y, .print=z, }
#define init_proto2(x,y,z,a,b)   { .name=x, .decode=y, .print=z, .sig_match=a, .sig_add=b, }
#define null_proto()             { /* all-zero by default */ }
struct proto {
	proc_decode		decode;

	/* Intelligent stream reassembly */
	proc_sdecode		sdecode;
	void			(*flow_ctor)(void *);
	void			(*flow_dtor)(void *);
	size_t			flow_len;

	/* preprocessors */
	struct notifier		preproc;

	/* linkages */
	struct proto_child	*children;
	struct proto		*next;

	/* Signature engine */
	proc_sig_add		sig_add;
	proc_sig_match		sig_match;
	struct sub_table	*sig_subs;

	/* Session save/restore */
	proc_session_free	free;
	proc_serialize		serialize;
	proc_deserialize	deserialize;

	/* Tagging */
	proc_tag		tag;

	/* print out a one line summary given a header */
	proc_dprint		print;

	/* Misc, meta data */
	const char		*name;

	/* Indexing data */
	struct field		*fields;
};

struct proto_table {
	struct proto *proto;
	struct proto_req *preq;
};

struct field;

void decode_load(void);
struct proto *decode_proto(const char *) __nonull(1);
struct proto *decode_subproto(const char *, unsigned int) __nonull(1);

extern const struct proto *pcap_dlt;

static inline struct alert *alert_best(struct alert *a1, struct alert *a2)
	__nonull(1, 2);
static inline struct alert *alert_best(struct alert *a1, struct alert *a2)
{
	if ( a1 == NULL )
		return a2;
	if ( a2 == NULL )
		return a1;
	if ( a2->priority < a1->priority )
		return a2;
	if ( a2->order < a1->order )
		return a2;
	return a1;
}

static inline void alert_tag(struct packet *p, struct alert *a, int tagl)
	__nonull(1, 2);
static inline void alert_tag(struct packet *p, struct alert *a, int tagl)
{
	if ( a && a->tag && tagl >= 0 ) {
		struct alert *orig_alert = p->alert;

		p->alert = a;

		if ( p->layer[tagl].proto->tag ) {
			p->layer[tagl].proto->tag(p, tagl, a->tag);
		}else{
			mesg(M_CRIT, "%s not taggable",
				p->layer[tagl].proto->name);
		}

		p->alert = orig_alert;
	}

	p->alert = alert_best(p->alert, a);
}

#endif /* __DECODE_HEADER_INCLUDED__ */
