/*
 * This file is part of dotscara
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 */
#ifndef __SYS_HEADER_INCLUDED__
#define __SYS_HEADER_INCLUDED__

#define DSE_NATIVE_ENDIAN	0
#define DSE_BIG_ENDIAN		1
#define DSE_LITTLE_ENDIAN	2

int __public sys_errno(void);
const char __public *sys_error(int);
const char __public *sys_err(void);
const char __public *sys_err2(const char *);

static inline u_int16_t __constfn sys_bswap16(u_int16_t x)
{
	return (u_int16_t)(
		(((u_int16_t)(x) & (u_int16_t)0x00ffU) << 8) |
	 	(((u_int16_t)(x) & (u_int16_t)0xff00U) >> 8));
}

static inline u_int32_t __constfn sys_bswap32(u_int32_t x)
{
	return (u_int32_t)(
		(((u_int32_t)(x) & (u_int32_t)0x000000ffUL) << 24) |
		(((u_int32_t)(x) & (u_int32_t)0x0000ff00UL) <<  8) |
		(((u_int32_t)(x) & (u_int32_t)0x00ff0000UL) >>  8) |
		(((u_int32_t)(x) & (u_int32_t)0xff000000UL) >> 24));
}

#if __BYTE_ORDER == __BIG_ENDIAN
#define sys_le32(x) sys_bswap32(x)
#define sys_le16(x) sys_bswap16(x)
#define sys_be32(x) (x)
#define sys_be16(x) (x)
#elif __BYTE_ORDER == __LITTLE_ENDIAN
#define sys_le32(x) (x)
#define sys_le16(x) (x)
#define sys_be32(x) sys_bswap32(x)
#define sys_be16(x) sys_bswap16(x)
#else
#error "What in hells name?!"
#endif

#endif /* __SYS_HEADER_INCLUDED__ */
