/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __ELOG_WRITE_HEADER_INCLUDED__
#define __ELOG_WRITE_HEADER_INCLUDED__

#include <dotscara/fobuf.h>

#define ALERT_FN "alert.elog"
#define ALERT_FNLEN 31

#define SPOOL_DEFAULT_MINS 60
#define SPOOL_DEFAULT_SIZE (512*1024)
#define SPOOL_DEFAULT_BUFFER (16*1024)

/* Number of available I/O vectors */
#define ALERT_VEC 6

/* Stormwall mode */
#define STORMWALL_NONE 0 /* stormwall comms disabled */
#define STORMWALL_WAIT 1 /* wait for stormwall to come alive */
#define STORMWALL_FAIL 2 /* fail immidietly if stormwall is not ready */

struct elog_spool_head {
	struct elog_spool *next, *prev;
};

struct elog_spool {
	struct elog_spool *next, *prev;

	/* size rotation */
	off_t max_bytes;
	off_t size;

	/* Current packet data */
	off_t tot_len;
	struct fvec iov[ALERT_VEC];
	struct elog_pkthdr ph;
	int io_len;

	/* state information */
	struct timeval tv;

	/* time rotation parameters */
	unsigned int max_time;

	/* Output buffer */
	struct fobuf buf;
	int nobuf;

	/* The files */
	char *alert_fn;
	char *rotate_fn;
	char *log_dir;

	/* stormwall stuff */
	char *fifo_fn;
	int fifo_fd;
	int stormwall;
};

static inline int spool_isempty(struct elog_spool *s)
{
	return (s->tv.tv_sec) ? 0 : 1;
}

/* Spool API */
struct elog_spool *spool_new(void);
int spool_set_buf(struct elog_spool *, size_t) __nonull(1);
void spool_link(struct elog_spool *) __nonull(1);
void spool_init_all(void);
void spool_rotate_all(void);
void spool_rotate_check(struct timeval *tv) __nonull(1);
void spool_delete(struct elog_spool *) __nonull(1);

#endif /* __ELOG_WRITE_HEADER_INCLUDED__ */
