/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __G_ETREE_MODEL_HEADER_INCLUDED__
#define __G_ETREE_MODEL_HEADER_INCLUDED__

#define ETREE_MODEL_TYPE (etree_model_get_type())
#define ETREE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), ETREE_MODEL_TYPE, EtreeModel))
#define IS_ETREE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), ETREE_MODEL_TYPE))

typedef struct _EtreeModel EtreeModel;
typedef struct _EtreeModelClass EtreeModelClass;

#define ETREE_ICON	0 /* pixbuf */
#define ETREE_PRIO	1 /* string */
#define ETREE_SRC	2 /* string */
#define ETREE_DST	3 /* string */
#define ETREE_GEN	4 /* string */
#define ETREE_MSG	5 /* string */
#define ETREE_TIME	6 /* string */
#define ETREE_SID	7 /* uint */
#define ETREE_MAX	8

GType etree_model_get_type(void);

EtreeModel *etree_model_new(void);
void etree_model_close(EtreeModel *e);
int etree_model_open(EtreeModel *e, void *elog, struct q_db *db);
int etree_model_get(EtreeModel *e, GtkTreeIter *iter,
			proc_elogfn cb, gpointer user_data);
void etree_model_show_all(EtreeModel *e);
void etree_model_clear(EtreeModel *e);
void etree_model_filter_away_sid(EtreeModel *e, u_int32_t sid);

struct _EtreeModel {
	GObject parent;

	/*< private >*/
	gint stamp;

	/* Query results */
	size_t num_pkts;
	idx_ofs_t *ofs;

	/* elog */
	void *efile;

	/* index */
	struct q_db *db;
};

struct _EtreeModelClass {
	GObjectClass parent_class;
};

static inline size_t etree_model_num_pkts(EtreeModel *e)
{
	g_return_val_if_fail(IS_ETREE_MODEL(e), 0);
	return e->num_pkts;
}

#endif /* __G_ETREE_MODEL_HEADER_INCLUDED__ */
