/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __NOTIFIER_HEADER_INCLUDED__
#define __NOTIFIER_HEADER_INCLUDED__

#define NOTIFY_PACKET		1 /* Called for every packet */
#define NOTIFY_PRE_ALERT	2 /* Called for all alerts (pre-token-bucket) */
#define NOTIFY_ALERT		3 /* Called for all alerts that pass */
#define NOTIFY_HUP		4 /* On receipt of SIGHUP */
#define NOTIFY_TIMER		5 /* Periodic timer, may never fire */
#define NOTIFY_USR1		6 /* On receipt of SIGUSR1 */
#define NOTIFY_USR2		7 /* On receipt of SIGUSR2 */
#define NOTIFY_MAX		8

/* Actually does packet preprocessing */
typedef void (*notifier_t)(struct packet *, int, void *);

/* Frees up the private data */
typedef void (*notifier_dtor_t)(void *);

struct notifier {
	unsigned short num;
	unsigned short max;
	struct notifier_chain *chain;
};

int notifier_add(struct notifier *, notifier_t, notifier_dtor_t, void *)
	__nonull(1, 2);
void notifier_delete(struct notifier *)
	__nonull(1);
void notifier_run(struct notifier *, struct packet *, int)
	__nonull(1);
void notifier_run_backwards(struct notifier *, struct packet *, int)
	__nonull(1);

int notify(int, notifier_t, notifier_dtor_t, void *)
	__nonull(2);
void __private notify_run(int, struct packet *, int);

#endif /* __NOTIFIER_HEADER_INCLUDED__ */
