/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco
 * This program is released under the terms of the GNU GPL version 2
 *
 * Functions:
 *  o serial_number() - get a per-capture-session unique id.
 *  o capdev_add() - register a capdev plugin
 *  o capdev_find() - find a capture type by name.
 *  o capdev_add() - called by plugins to add new capture types.
*/
#include <firestorm.h>
#include <loader.h>
#include <args.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <capture.h>

struct capdev *capdevs=NULL;
static serial_t the_serial;

void serial_number(struct packet *p)
{
	p->u.capture.serial = the_serial++;
}

void capdev_init_packet(struct packet *p, struct capture *cap)
{
	p->source = FPS_CAPTURE;
	p->u.capture.cap = cap;
	p->usage = 1;
	p->dtor = NULL;
}

/* plugins call this to register capdevs */
static void capdev_add(struct plugin *p, struct capdev *cap)
{
	for(; cap->name; cap++) {
		if ( capdev_find(cap->name) ) {
			mesg(M_DEBUG,"capdev: error: '%s' capdev is "
				"already registered", cap->name);
			continue;
		}
		cap->next=capdevs;
		capdevs=cap;
	}
}

void capdev_load(void)
{
	loader_find_syms("firestorm_capdevs", (loader_cb_t)capdev_add);
}

/* Find a capdev with the specified name */
struct __private capdev *capdev_find(const char *name)
{
	struct capdev *c;

	if ( !name )
		return NULL;

	for(c=capdevs; c; c=c->next) {
		if ( !strcmp(c->name, name) )
			return c;
	}

	return NULL;
}
