/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2004 Gianni Tedesco
 * This program is released under the terms of the GNU GPL version 2
*/

#include <firestorm.h>
#include <packet.h>
#include <capture.h>

#include <stdarg.h>
#include <unistd.h>

static int in_cleanup;
static char cleanup_err[1024];
static struct notifier cleaners;

void packet_reinject_prepare(struct packet *dst, struct packet *src)
{
	memset(dst, 0, sizeof(*dst));
	dst->source = src->source;
	dst->u.capture.cap = src->u.capture.cap;
	serial_number(dst);
	dst->time = src->time;
	dst->flags = (src->flags | FP_COPY | FP_WRITABLE) &
			~(FP_REAL|FP_PKTTYPE|FPMASK_PKTTYPE);
}

void firestorm_init(void)
{
	loader_init();
	notifier_init();
	decode_init();
	field_init();
	capture_init();
}

void notify_exit(notifier_t fn, void *priv)
{
	if ( !notifier_add(&cleaners, fn, NULL, priv) )
		firestorm_die_error("notifier_add()");
}

/* exit the application, calling all cleanup handlers
 * as we go */
void firestorm_exit(int code, const char *format, ...)
{
	va_list va;
	unsigned char ecode = M_INFO;

	/* check for illegal reentrancy condition */
	if ( in_cleanup ) {
		mesg(M_ERR,"exit: firestorm_exit() reentry!");
		exit(EXIT_FAILURE);
	}
	
	in_cleanup = 1;

	/* Format and print final error message */
	if ( format ) {
		va_start(va, format);
		vsnprintf(cleanup_err, sizeof(cleanup_err), format, va);
		va_end(va);
		if ( code == EXIT_FAILURE )
			ecode = M_ERR;
	}else{
		sprintf(cleanup_err, "No reason specified");
	}

	mesg(ecode,"exit: %s", cleanup_err);

	notifier_run_backwards(&cleaners, NULL, -1);
	
	mesg(M_DEBUG,"cleanup: exit with code %i", code);

	/* prevent runaway interactions */
	if ( code==EXIT_FAILURE ) {
		usleep(500000);
	}
	
	exit(code);
}
