/*
* This file is part of Firestorm NIDS
* Copyright (c) 2003,2004 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*/

#include <firestorm.h>

/* Convert a vector string in to an unsigned int */
size_t vtouint(struct fvec *v, unsigned int *u)
{
	unsigned char *chr = v->ptr;
	size_t i;

	*u = 0;

	for(i=0; i < v->len; i++) {
		unsigned int digit = *chr++ - '0';
		if ( digit >= 10 )
			break;

		*u *= 10;
		*u += digit;
	}

	return i;
}

/* Case-insensitive comparison of two vectors */
int vcasecmp(const struct fvec *v1, const struct fvec *v2)
{
	const char *end1, *end2;
	const char *p1, *p2;

	end1 = v1->ptr + v1->len;
	end2 = v2->ptr + v2->len;

	p1 = v1->ptr;
	p2 = v2->ptr;

	for(;;) {
		if ( p1==end1 && p2==end2 )
			return 0;

		if ( p1==end1 )
			return -1;

		if ( p2==end2 )
			return 1;

		if ( toupper(*p1) < toupper(*p2) )
			return -1;

		if ( toupper(*p1) > toupper(*p2) )
			return 1;

		p1++; p2++;
	}
	
}

/* Case-sensitive comparison of two vectors */
int vcmp(const struct fvec *v1, const struct fvec *v2)
{
	const char *end1, *end2;
	const char *p1, *p2;

	end1 = v1->ptr + v1->len;
	end2 = v2->ptr + v2->len;

	p1 = v1->ptr;
	p2 = v2->ptr;

	for(;;) {
		if ( p1==end1 && p2==end2 )
			return 0;

		if ( p1==end1 )
			return -1;

		if ( p2==end2 )
			return 1;

		if ( *p1 < *p2 )
			return -1;

		if ( *p1 > *p2 )
			return 1;

		p1++; p2++;
	}
	
}

/* Case-insensitive comparison of a string and a vector */
int vstrcmp(const struct fvec *v1, const char *str)
{
	struct fvec v2;

	v2.ptr = (void *)str;
	v2.len = strlen(str);

	return vcasecmp(v1, &v2);
}
