/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2003,2004 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
*/
#include <firestorm.h>
#include <stdarg.h>
#include <mesg.h>

/* Default mesg handler just prints to stdout */
static void mesg_default(unsigned char code, const char *str, size_t len)
{
	printf("%s\n", str);
}

/* API for changing the mesg backend */
static mesg_t m = mesg_default;
void mesg_set_backend(mesg_t fn)
{
	if ( fn == NULL )
		m = mesg_default;
	else
		m = fn;
}

static char *abuf;
static size_t abuflen;

/* Print out a firestorm internal system log message */
void mesg(unsigned char code, const char *fmt, ...)
{
	int len;
	va_list va;
	char *new;

again:
	va_start(va, fmt);

	len = vsnprintf(abuf, abuflen, fmt, va);
	if ( len < 0 ) /* bug in old glibc */
		len = 0;
	if ( (size_t)len < abuflen )
		goto done;

	new = realloc(abuf, len + 1);
	if ( new == NULL )
		goto done;

	abuf = new;
	abuflen = len + 1;
	goto again;

done:
	(*m)(code, abuf, (size_t)len);
	va_end(va);
}
