/*
* This file is part of Firestorm NIDS
* Copyright (c) 2003,2004 Gianni Tedesco
* Released under the terms of the GNU GPL version 2
*
* This code takes the big idx file with many indexes in and allows you
* to query it etc...
*/

#include <firestorm.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <unistd.h>

#include <idx.h>
#include <q_idx.h>

static int cmp(const void *a, const void *b)
{
	const struct q_idx *aa=a, *bb=b;
	return strcmp(aa->name, bb->name);
}

struct q_db *q_idx_open(int fd)
{
	struct q_db *db;
	struct stat st;
	struct idx_hdr *hdr;
	struct idx_index *idx;
	unsigned int i;
	char *map;

	if ( fstat(fd, &st) )
		return NULL;

	if ( st.st_size < (off_t)sizeof(*hdr) )
		return NULL;

	if ( (map=mmap(NULL, st.st_size, PROT_READ,
		MAP_SHARED, fd, 0))==MAP_FAILED )
		return NULL;

#if HAVE_MADVISE && defined(MADV_RANDOM)
	madvise(map, st.st_size, MADV_RANDOM);
#endif

	hdr = (struct idx_hdr *)map;
	idx = (struct idx_index *)(map + sizeof(*hdr));

	if ( st.st_size < (off_t)(sizeof(*hdr) + hdr->num * sizeof(*idx)) )
		return NULL;

	/* Check all indexes */
	for(i=0; i<hdr->num; i++) {
		if ( idx[i].name[IDX_NAMELEN-1] != '\0' )
			return NULL;

		if ( (off_t)(idx[i].ofs + idx[i].len) > st.st_size )
			return NULL;
	}

	/* allocate q_db then num * q_idx all at once */
	db = calloc(1, sizeof(struct q_db) +
		(hdr->num * sizeof(struct q_idx)));

	db->num_idx = hdr->num;
	db->map = map;
	db->end = map + st.st_size;
	for(i=0; i<db->num_idx; i++) {
		db->idx[i].name = idx[i].name;
		db->idx[i].map = map + idx[i].ofs;
		db->idx[i].end = db->idx[i].map + idx[i].len;
		db->idx[i].num_keys = idx[i].num_keys;
		db->idx[i].num_vals = idx[i].num_vals;
		db->idx[i].type = idx[i].type;
	}

	/* sort for bsearch */
	qsort(db->idx, db->num_idx, sizeof(*db->idx), cmp);

	return db;
}

struct q_idx *q_idx_get(struct q_db *q, const char *name)
{
	struct q_idx key;
	key.name=name;
	return bsearch(&key, q->idx, q->num_idx, sizeof(*q->idx), cmp);
}

void q_idx_close(struct q_db *q)
{
	munmap(q->map, q->end - q->map);
	free(q);
}
