/*
* This file is part of Firestorm NIDS
* Copyright (c) 2003,2004 Gianni Tedesco
* Released under the terms of the GNU GPL version 2
*
* Code for querying skunk indexes.
*/

#include <firestorm.h>
#include <dotscara/fobuf.h>
#include <dotscara/mpool.h>
#include <idx.h>
#include <q_idx.h>
#include <q_skunk.h>

#if 1
#define dprintf(...)
#else
#define dprintf printf
#endif

typedef int (*cmpfn)(const void *s1, const void *s2, size_t n);

static int memcasecmp(const void *s1, const void *s2, size_t n)
{
	size_t i;
	u_int8_t d1, d2;

	for(i=0; i < n; i++,s1++,s2++) {
		d1 = toupper(*(u_int8_t *)s1);
		d2 = toupper(*(u_int8_t *)s2);
		if ( d1 < d2 )
			return -1;
		if ( d1 > d2 )
			return 1;
	}

	return 0;
}

static int qry(struct q_idx *idx, struct q_res *result,
		struct fvec *key, int nocase)
{
	struct skunk_table *t;
	struct skunk_bucket *b;
	idx_ofs_t i, dist=0;
	idx_hash_t hash;
	idx_hash_t bucket;
	cmpfn cmp;

	result->num_keys = 0;
	result->num_vals = 0;

	if ( nocase ) {
		hash = skunk_hash_nocase(key->ptr, key->len);
		cmp = memcasecmp;
	}else{
		hash = skunk_hash(key->ptr, key->len);
		cmp = memcmp;
	}
	bucket = hash & (SKUNK_HASHSZ-1);

	t = ((struct skunk_table *)idx->map) + bucket;
	if ( !t->ofs || !t->len ) {
		dprintf("Nothing in primary hash table\n");
		return 1;
	}

	if ( idx->map + t->ofs > idx->end ) {
		dprintf("Whoah, primary hash entry goes past the end\n");
		return 0;
	}

	i = (hash ^ key->len) % t->len;
	for(;;) {
		b = ((struct skunk_bucket *)(idx->map + t->ofs)) + i;

		if ( (void *)b > idx->end )
			return 0;

		if ( b->hash == hash && b->klen == key->len ) {
			char *record = (char *)(idx->map + b->kofs);

			if ( idx->map + b->kofs + b->klen > idx->end )
				return 0;
			if ( idx->map + b->vofs + b->vlen > idx->end )
				return 0;

			if ( (*cmp)(record, key->ptr, key->len) != 0 )
				goto not_this;

			result->num_keys = 1;
			result->num_vals = b->vlen;
			result->val = idx->map + b->vofs;
			result->node = &result->dummy;
			result->dummy.num = result->num_vals;
			return 1;
		}
		dprintf("%u %u", b->hash, b->klen);
not_this:
		if ( ++dist > t->len )
			break;

		i = (i+1) % t->len;
	}

	dprintf("NOWT\n");
	return 1;
}

int q_skunk_query(struct q_idx *idx, struct q_res *result, struct fvec *key)
{
	return qry(idx, result, key, 0);
}

int q_skunk_nocase(struct q_idx *idx, struct q_res *result, struct fvec *key)
{
	return qry(idx, result, key, 1);
}
